#!/usr/local/bin/perl
#
# This script searches a gopher index
#
# then Searches a set of clusters...
#

######################################################################
# Parameters below:
#

$IndexHost  = shift;
$IndexPort  = shift;
$IndexPath  = shift;
$ClusterDir = shift;

######################################################################

$searchwords = shift;

chdir($ClusterDir) || die "Cannot change to $Clusterdir";

if (!$searchwords) {
    $searchwords = <>;
    $searchwords =~ s/[\r\n]*//g;
}

#
# Execute the search here, get results...
#
if ($searchwords) {

    &GopenServer("$IndexHost", $IndexPort);
    &Gsend("$IndexPath\t$searchwords\r\n");
    while (1) {

	$_ = &Grecv;
	last if /^\./;
	($title, $path, $host, $port, $end) = split(/\t/);
	$Hits{"$path\t$host\t$port"} = 1;
    }
    &GcloseServer;
    
}


#
# Open ClusterTitles file, get the fancy names...
#
open(ctitle, "<ClusterTitles");
$i = 0;

while (<ctitle>) {
    chop;
    $ctitle[$i] = $_;
    $i++;
}
close(ctitle);

#
# Get some stats
#
open(lines, "wc -l *|");

$maxcluster = 0;

while (<lines>) {
    chop;

    next if (! m/([\d]+)[ \t]+(cluster[\d]+)$/);
    $fname = $2;
    $lines = $1;
    $clusternum = $fname;
    $clusternum =~ s/[^\d]//g;
    $ClusterSize[$clusternum] = $lines;
    $Numdocs += $lines;
}

#
# select a gridsize that will contain the largest cluster..
#

$cf = 0;
@validclusters = ();

foreach $i (0..$#ClusterSize) {

    #
    # Test to see if an item that fits is in the cluster..
    # Eliminate clusters that don't contain found items..
    #
    open(cfile, "<cluster$i") || die "No cluster file...";
    $numhits = 0;
    while (<cfile>) {
	chop;

	($title, $path, $host, $port, $end) = split(/\t/);
	
	if ($Hits{"$path\t$host\t$port"}) {
	    $numhits = 1;
	}
    }
    close(cfile);

    next if (! $numhits);
    push(@validclusters, $i);
    $cf++;
    $maxcluster = $ClusterSize[$i] if ($ClusterSize[$i] > $maxcluster);

}

$gridsize = &MaxWidth($maxcluster);

$square=0;
while (1) {
    $square ++;
    last if (($square * $square) > $#validclusters);
}

$square = 2 if ($square < 2);

$t = 0;

foreach $i (@validclusters) {
    $startx =  4 * $gridsize * (($t % ($square)) - $square/2);
    $starty =  1.5 * $gridsize * (($t) - ($t % ($square)) +1 );

    if ($ctitle[$i]) {
	&OutputClusterTitle($ctitle[$i], $startx, $starty);
    } else {
	&OutputClusterTitle("", $startx, $starty);
    }
    &OutputSpiral("cluster$i", $ClusterSize[$i], $startx, $starty);
    $t++;
    
}

sub OutputClusterTitle {
    ($title, $x, $y) = @_;
    print "+INFO: 0$title\txx\txx\t70\r\n";
    print "+3D:\r\n";
    print " Location: $y, -1500.0, $x\r\n";
    print " Orientation: 0, -90, 0\r\n";
    print " Scale: 2, 2, 2\r\n";

}


#
# Make a stonehenge...
# 

sub OldOutputSpiral {
    ($file, $numitems, $xcenter, $ycenter) =  @_;
    $delta = ((2 * 3.14159) / $numitems);
    $angle = -1.57;

    open(thefile, "<$file");

    #
    # Put a cool title above the cluster..
    #

    $radius = $numitems * 130.0;
    while (<thefile>) {
	chop;
	print "+INFO: ";
	print;
	print "\r\n";
	print "+3D:\r\n";
	print " Location: ";
	print sin($angle) * $radius + $ycenter;
	print ", -2.0, ";
	print cos($angle) * $radius + $xcenter;
	print "\r\n";
	print " Orientation: 0, ";
	print (((3.14159) + $angle ) *  57.29578);
	print ", 0\r\n";

	#
	# If the item is in our hit list, make it bigger...
	# 
	($title, $path, $host, $port, $end) = split(/\t/);

	if ($Hits{"$path\t$host\t$port"}) {
	    print " Scale: 1, 2, 1\r\n";
	}
	$angle += $delta;
    }
}


sub OutputSpiral {
    ($file, $numitems, $xcenter, $ycenter) =  @_;
    $angle = 0.1;

    $PI = 3.14159;
    $delta = ($PI/3);
    $deltaLimit    = $PI * 2 - 0.1;
    $radius  = 800;

    open(thefile, "<$file");

    #
    # Put a cool title above the cluster..
    #
    $i = 0;
    while (<thefile>) {
	$i++;
	chop;
	($title, $path, $host, $port, $end) = split(/\t/);


	print "+INFO: ";
	if ($Hits{"$path\t$host\t$port"}) {  # Indent hit..
	    s/^(.)/$1  /;
	}
	print;
	print "\r\n";
	print "+3D:\r\n";
	print " Location: ";
	print sin($angle) * $radius + $ycenter;
	print ", -2.0, ";
	print cos($angle) * $radius + $xcenter;
	print "\r\n";
	print " Orientation: 0, ";
	print (((3.14159) + $angle ) *  57.29578);
	print ", 0\r\n";

	#
	# If the item is in our hit list, make it bigger...
	# 

	if ($Hits{"$path\t$host\t$port"}) {
	    print " Scale: 1.5, 2, 1.5\r\n";
	}
	$angle += $delta;
	if ($angle > $deltaLimit) {
	    # Start a new circle

	    $angle =  0.1;
	    $radius += 700;
	    $delta = $delta * ($radius - 700.0)/ $radius;
	}	    
    }

}


sub GopenServer {

    local($server,$port) = @_;
    local($type, $name, $aliases, $len, $saddr);

    $sockaddr = 'S n a4 x8';
    ($name, $aliases, $proto) = getprotobyname('tcp');
    (($name, $aliases, $type, $len, $saddr) = gethostbyname($server))
	|| &Gabort("3Can't get address of: $server");
    $sin = pack($sockaddr, 2, $port, $saddr);
    socket(GSERVER, 2, 1, 0) || &Gabort("Can't create socket: $!");
    connect(GSERVER, $sin)   || &Gabort("Can't connect to server: $!");
    select(GSERVER); $| = 1; select(STDOUT); $| = 1;
}

sub GcloseServer {
    close(GSERVER);
}

sub Gsend { 
    print "send -> |$_[0]|\n" if (defined($Gdebug));
    print GSERVER "$_[0]\r\n"; 
}

sub Grecv { 
    local ($_); 
    $_= <GSERVER>; 
    s/\n$//;
    s/\r$//;
    print "recv -> |$_|\n" if (defined($Gdebug));
    return $_; 
}

sub Gabort {
    local ($mesg) = @_;
    print "0$mesg\t\terror.host\t70\r\n";
    exit;
}

# 
# Finds the maximum width of the largest cluster...
#

sub MaxWidth {
    local($sz) = @_;

    $angle = 0.1;

    $PI = 3.14159;
    $delta = ($PI/3);
    $deltaLimit    = $PI * 2 - 0.1;
    $radius  = 800;

    while ($sz > 0) {
	$angle += $delta;
	if ($angle > $deltaLimit) {
	    # Start a new circle

	    $angle =  0.1;
	    $radius += 700;
	    $delta = $delta * ($radius - 700.0)/ $radius;
	}
	$sz--;
    }

    return($radius);
}
