#!/usr/local/bin/perl

require 'getopts.pl';

&Getopts('h:p:d:');

$Hostname = $opt_h;
$Port = $opt_p;
$GopherDir = $opt_d;

if (!($Port) || !($Hostname) || !$GopherDir) {
    print "I need the following flags:\n\n";
    print "  -h the gopher host where the wais directory resides\n";
    print "  -p the gopher port where the wais directory resides\n";
    print "  -d the gopher-data directory (used to strip the full paths in WAIS\n";
    print "     make sure that you omit the trailing slash...\n";
    exit;
}

$HUGE = 100000;
$| = 1;

($WAIScat) = shift || die "Need to specify a WAIS .cat file...";
print "Processing $WAIScat\n";
open(waisf, "<$WAIScat") || die "Cannot open $WAIScat";
$step = 0;

%Dict = ();
$MaxDict = 1;

sub AddtoDict {
    local($word) = @_;

    $val = $Dict{"$word"};
    return $val if ($val > 0);
	
    $Dict{"$word"} = $MaxDict++;
    return($MaxDict-1);
}

#
# read documents
#

$NDoc = 0;
$NWord = 0;

while (<waisf>) {
    chop;
    if (/^Headline: (.*)$/) {
	$headline = $1;
    }
    next if (! /^DocID: ([\d]+) ([\d]+) (.*)$/);
    $startbyte = $1;
    $endbyte = $2;
    $doc = $3;
    next if ($startbyte == $endbyte);
    print "Processing $doc, $startbyte, $endbyte\n";
    open (DOC, "ClusterWords.pl \"$doc\" $startbyte $endbyte|");
    undef(@tmp);
    while (<DOC>) {
	chop;
	($Word, $Freq) = split;

	$Word = &AddtoDict($Word); # Use word number instead of text...
	push(@tmp, ($Word));
	$NWord += $Freq;
	$NWordInD[$NDoc] += $Freq;
	$NWordInC[$NDoc] += $Freq;

	$WFreq[$Word] += $Freq;
	$WFreqInD{$NDoc,$Word} += $Freq;
	$WFreqInC{$NDoc,$Word} += $Freq;

    }
    $NDocs[$NDoc] = 1;
    $DocList[$NDoc] = "$NDoc";
    $WordList[$NDoc] = join($;, @tmp);
    $doc =~ s/$GopherDir//;
    $clusterid[$NDoc] = "0$headline\tR$startbyte-$endbyte-$doc";
    $ID[$NDoc++] = "$headline";
    if ($NWordInD[$NDoc-1] == 0) {
	$NDoc--;
    }
    close(DOC);

}

foreach $w (0..$#WFreq) {
    $InvWFreq[$w] = $WFreq[$w]/$NWord;
}

undef(%Dict);
#
# Initialize
#
print "Initializing Intra array....\n";
for ($c = 0; $c < $NDoc; $c++) {
    # P(c|d)
    $Intra[$c] = &UnitIntra($c);
}
print "Initializing Pmatrix....\n";
$starttime = time();
for ($i = 0; $i< $NDoc - 1; $i++) {
    $t = time();
    for ($j = $i + 1; $j < $NDoc; $j++) {
	# P(c1 \cup c2|d)
	next if ($i == $j);
	$Pmatrix{$i, $j} = &InitialMergeIntra($i, $j);
    }
    $t = time() - $t;
    print "Processed Doc #$i Time $t\n";
}
$starttime = time() - $starttime;
print "Pmatrix init took $starttime\n";
#
# Clustering

for ($step = 1; $step < $NDoc; $step++) {
    ($c1, $c2, $prob) = &FindClosestPair();

    printf STDOUT ("%d\t%d\t%s\t%s\t%e\n", $c1, $c2, $ID[$c1], $ID[$c2], $prob);
    print "----------------------------------------------------------------------\n";
    print "---Adding:\n$clusterid[$c2]\n---to:\n";
    print "$clusterid[$c1]\n";
    print "----------------------------------------------------------------------\n";
    $clusterid[$c1] = $clusterid[$c1] . "\n" . $clusterid[$c2];
    $clusterid[$c2] = "";
    &MergePair($c1, $c2);

    last if (($prob < -5) && ($step > ($NDoc/2)));

}

#
# Output the clusters to a file here...
#

$cnum = 0;

foreach $i (0..$#clusterid) {
    if (length($clusterid[$i]) > 1) {
	$fn = "cluster" . $cnum;
	open(fn, ">$fn");
	$clusterid[$i] =~ s/\n/\t$Hostname\t$Port\n/g;
	print fn "$clusterid[$i]";
	close(fn);
	$cnum++;
    }
}


sub UnitIntra { # calculate P(c|d)
    local ($c) = @_;
    local ($out, $tmp, $d, $w);
    
    $out = 0.0;
    foreach $d (split($;, $DocList[$c])) {
	$tmp = 0.0;
	foreach $w (split($;, $WordList[$c])) {
	    $tmp += ($WFreqInD{$d,$w}/$NWordInD[$d]) *
		($WFreqInC{$c,$w}/$NWordInC[$c]) /
		    ($InvWFreq[$w]);
	}
	$out += log($tmp);

    }
    return($out);
}


sub MergeIntra {
    local($c1, $c2) = @_;
    local($out, $tmp, $d, $w);

    $totalwords = $NWordInC[$c1] + $NWordInC[$c2];
    $out = 0.0;
    foreach $d (&Union($DocList[$c1], 
		       $DocList[$c2])) {
	$tmp = 0.0;
	foreach $w (&Union($WordList[$c1], $WordList[$c2])) {
	    $tmp += ($WFreqInD{$d,$w}/$NWordInD[$d]) *
		   (($WFreqInC{$c1,$w} + $WFreqInC{$c2,$w}) /
		    ($totalwords)) /
		     ($InvWFreq[$w]);
	}
	if ($tmp != 0.0) {
	    $out += log($tmp);
	} else {
	    $tmp = 0.0;
	}
    }
    return($out);
}

sub InitialMergeIntra {
    local($c1, $c2) = @_;
    local($out, $tmp, $d, $w);

    $out = 0.0;
    $totalwords = $NWordInC[$c1] + $NWordInC[$c2];

	$tmp = 0.0;
	$nwc1 = $NWordInD[$c1];
	$nwc2 = $NWordInD[$c2];

	foreach $w (&Union($WordList[$c1], $WordList[$c2])) {
	    $tmp1 += ($WFreqInD{$c1,$w}/$nwc1) *
		   (($WFreqInC{$c1,$w} + $WFreqInC{$c2,$w}) /
		    ($totalwords))/ $InvWFreq[$w];

	    $tmp2 += ($WFreqInD{$c2,$w}/$nwc2) *
		   (($WFreqInC{$c1,$w} + $WFreqInC{$c2,$w}) /
		    ($totalwords))/ $InvWFreq[$w];


	}
    $out += log($tmp1) if ($tmp1 != 0.0);
    $out += log($tmp2) if ($tmp2 != 0.0);

    return($out);
}


sub FindClosestPair {
    local($i, $j);
    local($c1, $c2, $maxprob, $prob);

    $maxprob = - $HUGE;
    for ($i = 0; $i < $NDoc - 1 ; $i++) {
	if ($NDocs[$i] > 0) {
	    for ($j = $i + 1; $j < $NDoc; $j++) {
		if ($NDocs[$j] > 0) {
		    
		    $prob = $Pmatrix{$i,$j} - $Intra[$i] - $Intra[$j];
	    
		    if ($prob > $maxprob) {
			$c1 = $i;
			$c2 = $j;
			$maxprob = $prob;
		    }
		}
	    }
	}
    }
    return(($c1, $c2, $maxprob));
}

sub MergePair {
    local($c1, $c2) = @_;
    local($w,$c,$i,@tmp, $c_r, $c_l);

    $NWordInC[$c1] += $NWordInC[$c2];
    foreach $w (split($;, $WordList[$c2])) {
	$WFreqInC{$c1, $w} += $WFreqInC{$c2, $w};
    }
    
    @tmp = &Union($WordList[$c1], $WordList[$c2]);
    undef($WordList[$c2]);

    $WordList[$c1] = join($;, @tmp);


    @tmp = split($;, $DocList[$c1]);
    push(@tmp, split($;, $DocList[$c2]));
    undef($DocList[$c2]);

    $DocList[$c1] = join($;, @tmp);
    $NDocs[$c1] += $NDocs[$c2];
    $NDocs[$c2] = 0;

    $Intra[$c1] = &UnitIntra($c1);
    for ($i = 0; $i < $NDoc; $i++) {
	if (($i != $c1) && ($NDocs[$i] > 0)) {
	    if ($i < $c1) {
		$c_l = $i;
		$c_r = $c1;
	    } else {
		$c_l = $c1;
		$c_r = $i;
	    }
	    $Pmatrix{$c_l, $c_r} = &MergeIntra($c_l, $c_r);
	}
    }
}

sub Union {
    local($list1, $list2) = @_;
    local(@tmp, %freq);

    %freq = "";

    @tmp = split($;, $list1);
    push(@tmp, split($;, $list2));
	 
    foreach $i (@tmp) {
	$freq{$i}++;
    }

    return(keys(%freq));
}


		    
	   
