/*
 *        Name: GOPHERC REXX
 *              CMS Gopher client function as a CMS Pipelines stage
 *      Author: Rick Troth, Rice Univ, I/S VM Systems Support
 *        Date: 1992-Mar-06
 *
 *        Note: Path/selector string supplied may be either the third
 *              positional parameter or may be the first record of the
 *              input stream.   It is recommended that it be on the
 *              input stream,  or you run the risk of an open-paren
 *              "("  in the path/selector being misinterpreted as
 *              the CMS option delimiter.
 */


Parse Arg host port path "(" opts

trans = 0
Do While opts ^= ""
    Parse Var opts op opts
    Upper op
    Select  /*  op  */
        When Abbrev("TRANSLATE",op,4) Then trans = 1
        When Abbrev("NOTRANSLATE",op,6) Then trans = 0
        Otherwise Say "Unrecognized option" op
        End  /*  Select  op  */
    End  /*  Do  While  */


If trans Then crlf = '0D25'x
         Else crlf = '0D0A'x


/*  Call STATUS "Connecting ... press ENTER twice to abort"  */
Call STATUS "Connecting to" host "port" port

/*
**   Initialize RXSOCKET
*/
maxdesc = Socket('Initialize', 'Gopher')
If maxdesc = "-1" Then Do
    If errno ^= "ESUBTASKALREADYACTIVE" Then Do
      Call STATUS "RXSOCKET subfunction INITIALIZE returned error" errno
        Exit
        End  /*  If  ..  Do  */
    rc = Socket('Terminate')
    maxdesc = Socket('Initialize', 'Gopher')
    If maxdesc = "-1" Then Do
      Call STATUS "RXSOCKET subfunction INITIALIZE returned error" errno
        Exit
        End  /*  If  ..  Do  */
    End  /*  If  ..  Do  */


/*
**   Request a new socket descriptor (TCP protocol)
*/
socket = Socket('Socket', 'AF_INET', 'Sock_Stream')
If socket = "-1" Then Do
    Call STATUS "RXSOCKET subfunction SOCKET returned error" errno
    Exit
    End  /*  If  ..  Do  */


If trans Then Do
    /*
     *   Enable ASCII<->EBCDIC Translation Option
     */
    rc = Socket('SetSockOpt', socket, 'SOL_SOCKET', 'SO_EBCDIC', 1)
    If rc = "-1" Then Do
      Call STATUS "RXSOCKET subfunction SetSockOpt returned error" errno
        Exit
        End  /*  If  ..  Do  */
    End  /*  If  ..  Do  */


/*
**   Connect to the server
*/
Parse Var host h1 '.' h2 '.' h3 '.' h4 '.' .
If  Datatype(h1,'N') &,
    Datatype(h2,'N') &,
    Datatype(h3,'N') &,
    Datatype(h4,'N')    Then
    hisaddr = d2c(h1) || d2c(h2) || d2c(h3) || d2c(h4)
Else Do
    hisaddr = Socket('GetHostByName', host)
    If hisaddr = "-1" Then Do
   Call STATUS "RXSOCKET subfunction GETHOSTBYNAME returned error" errno
        Exit
        End  /*  If  ..  Do  */
    End  /*  Else  Do  */

/*  Parse Var hisaddr h1 +1 h2 +1 h3 +1 h4 +1 .  */

name = AF_INET || Htons(port) || hisaddr


/*
**   Set this socket to non-blocking mode
*/
rc = Socket('Ioctl', socket, 'FIONBIO', 1)
If rc="-1" Then Do
    Call STATUS "RXSOCKET subfunction IOCTL returned error" errno
    Exit
    End  /*  If  ..  Do  */


/*
**   Connect to the host
*/
rc = Socket('Connect', socket, name)
If rc = "-1" Then ,
   If errno ^= "EINPROGRESS" Then Do
    Call STATUS "RXSOCKET subfunction CONNECT returned error" errno
    If errno = "ECONNREFUSED" Then ,
       Call STATUS "Connect to host" host "port" port "REFUSED"
    Return
    End  /*  If  ..  Do  */

rc = FD_ZERO('readmask')
rc = FD_ZERO('writemask')
rc = FD_SET(socket,'writemask')
rc = FD_SET('0', 'readmask')

rc = Socket('Select', socket+1, 'readmask', 'writemask', 0, 20)
If rc="-1" Then Do
    Call STATUS "RXSOCKET subfunction Select returned error" errno
    Exit
    End  /*  If  ..  Do  */

If FD_ISSET('0', 'readmask')<>0 Then Do
   rc = Socket('Close', socket)
   Call STATUS "Connection canceled by user request"
   Exit
   End

If FD_ISSET(socket, 'writemask')=0 Then Do
   rc = Socket('Close', socket)
   Call STATUS "Connection canceled by TIMEOUT"
   Exit
   end


/* Return to standard mode
rc = Socket('Ioctl', socket, 'FIONBIO', 0)
*/


If path = "" Then 'READTO PATH'
             Else 'CALLPIPE VAR PATH | A2E | VAR PATH'
/*
**   Send the "query" to the server
*/
bytes_out = Socket('Write', socket, path || crlf)
If bytes_out = "-1" Then Do
    If errno = 'EPIPE' Then ,
       Call STATUS "Connect to host" host "port" port "REFUSED"
    Else Call STATUS "RXSOCKET subfunction WRITE returned error" errno
    Exit
    End  /*  If  ..  Do  */


Call STATUS "Reading ... press ENTER twice to abort"
/*
**   Loop, reading response and sending to the next pipeline stage
*/
Do Forever
    /* Set up to trap console ENTER key */
    rc = FD_ZERO('readmask')
    rc = FD_SET('0','readmask')
    rc = FD_SET(socket,'readmask')

    /* Wait for something to happen */
    rc = Socket('Select',socket+1,'readmask',0,0)
    /* If console input, clear 'socket' and return */
    If FD_ISSET('0','readmask')<>0 Then Do
        Call STATUS "Request ABORTED"
        rc = Socket('Close', socket)
        Exit
        End


    bytes_in = Socket('Read', socket, 'pack')
    If bytes_in = "-1" Then
        Call STATUS "RXSOCKET subfunction READ returned error" errno
    If bytes_in < 1 Then Leave
    'OUTPUT' pack
/*
    If (first<>'FIRST' & i//10=0) Then Do
       msg = Right(i,4,'0')
       ws = '00C3'x||sba(1,12)||'1D68'x||msg ,
            ||'1D60'x||"(Reading    ... press ENTER twice to abort)"
       'PIPE VAR WS | FULLSCR NOREAD | VAR RS'
    End
*/
    End


/*
**   All done, relinquish our socket descriptor
*/
rc = Socket('Close', socket)
If rc = "-1" Then Do
    Call STATUS "RXSOCKET subfunction CLOSE returned error" errno
    rc = Socket('Terminate')
    Exit
    End


/*
**   Tell RXSOCKET that we are done with this IUCV path
*/
rc = Socket('Terminate')
If rc = "-1" Then Do
    Call STATUS "RXSOCKET subfunction TERMINATE returned error" errno
    Exit
    End


/* an empty line as last in status stream indicates success */
Call STATUS " "


Exit



/* --------------------------------------------------------------- ERROR
**   Error routine
*/
Error:
   Arg subfunction, error_text

   path = ""
   rc = Socket('Path', 'path')

   Say "RXSOCKET subfunction" subfunction "returned errno:" error_text ,
      "for path:" path

   rc = Socket('Close', socket)
   If rc = "-1" Then Say "CLOSE:" errno

   rc = Socket('Terminate')
   If rc = "-1" Then Say "TERMINATE:" errno

   Exit


/* -------------------------------------------------------------- SYNTAX
**   Error routine
*/
SYNTAX:
   src = rc
   rc = Socket('Close', socket)
   rc = Socket('Terminate')

   Parse Source . . $Fn $Ft $Fm .
   Say 'Syntax error signaled on line' sigl 'in file' $Fn $Ft $Fm
   Say ""
   Say "Source line in error:"
   Say Sourceline(sigl)
   Say "Error description:" Errortext(src)
   Say ""

   Exit


/* -------------------------------------------------------------- STATUS
 * Write "status messages" to the secondary stream.
 * This routine saves&restores the current stream selection,
 * although there are no other streams presently used besides 1 and 0.
 */
STATUS:   Procedure
Parse Arg string

/* note the current stream (should be zero) */
'STREAMNO OUTPUT'
If rc < 0 Then Return
stream = rc

/* select secondary stream and output the string */
'SELECT OUTPUT 1'
If rc ^= 0 Then Return
'OUTPUT' string
'SELECT OUTPUT' stream

Return





