/*
 *        Name: GOPHERD REXX
 *              CMS Gopher Server main pipeline stage
 *      Author: Rick Troth, Rice Univ, I/S VM Systems Support
 *        Date: 1992-Apr-21
 *
 *       Calls:
 *              GOPHERDM REXX     -- to build menus from FILELISTs
 *
 *        Note:
 *              There is some inconsistency in the syle used for
 *              variable names in this code.   The author prefers
 *              lower case for variables,  capitalize first letter
 *              of REXX internal functions,  and capitalize all of
 *              quoted commands.   But convention in some TCP/IP
 *              support code uses different style with mixed case
 *              variable names where the first letter of each sig-
 *              nificant part of the name is capitalized.   Sorry
 *              for the confusion.    :-(
 */

Parse Upper Arg host port root client . "(" .
Say "HOST" host "PORT" port "ROOT" root "CLIENT" client

Parse Var query query '0A'x .   /* ASCII LF */
Parse Var query query '0D'x .   /* ASCII CR */

'CALLPIPE *: | A2E | VAR QUERY'
Say "Request:" query
If query = "" Then query = "1/"
Parse Var query 1 type 2 path

file = gresolve(root path)
Say "File:" file
Parse Var file fn ft fm menu .
pipe = pipespec(file)
Say "Pipe:" pipe

Select  /*  type  */
    When Left(pipe,1) = "-" Then pipe = 'LITERAL' pipe,
                                        '| APPEND LITERAL .',
                                        '| APPEND LITERAL .',
                                        '| E2A | SPEC 1-* 1 x0D0A NEXT'
    When type = "0" Then pipe = pipe ,          /* plain text */
                                '| APPEND LITERAL .',
                                '| APPEND LITERAL .',
                                '| E2A | SPEC 1-* 1 x0D0A NEXT'
    When type = "1" Then pipe = pipe ,          /* menu */
                                '| GOPHERDM' host port fn path,
                                '| APPEND LITERAL .',
                                '| APPEND LITERAL .',
                                '| E2A | SPEC 1-* 1 x0D0A NEXT'
    When type = "p" Then pipe = pipe ,          /* PostScript */
                                '| E2A | SPEC 1-* 1 x0D0A NEXT'
    When type = "s" Then nop                    /* sound */
    When type = "i" Then nop                    /* image (binary) */
    When type = "r" Then pipe = pipe ,          /* record oriented */
                                '| BLOCK 65531 CMS'
    Otherwise pipe = 'LITERAL - type' type 'is unknown',
                     '| APPEND LITERAL .',
                     '| APPEND LITERAL .',
                     '| E2A | SPEC 1-* 1 x0D0A NEXT'
    End  /*  Select  type  */

'CALLPIPE' pipe '| FBLOCK 1280 | *:'

Exit



/* ------------------------------------------------------------ GRESOLVE
 * Given a "path", search relavent FILELISTs until a fn/ft/fm
 * is found for that path-style file name.
 */
GRESOLVE: Procedure Expose emsg
Parse Arg menu path
Say "Resolving:" path "from" menu

/* strip off leading slashes (fast) */
Do While Left(path,1) = "/"; path = Substr(path,2); End
/* we could leave this out,  but recursion is slower */

Parse Var path nick "/" rest
nick = Strip(nick)
rest = Strip(rest)

If nick = "" & rest = "" Then Return menu "FILELIST *" menu
If nick = "" Then Return gresolve(menu rest)    /*   now impossible   */
nick = Translate(nick)                          /*  because of above  */

'CALLPIPE DISK' menu 'FILELIST * | STEM STEM.'
/*  Say stem.0 "lines found in" menu "FILELIST"  */
Do i = 1 to stem.0
    If Left(stem.i,1) ^= " " Then Iterate
    Parse Upper Var stem.i fn ft fm part '05'x .
    If ft = "" Then Iterate
/*  If ft = "" Then ft = "FILELIST"  */
    If fm = "" Then fm = "*"
    If part = "" Then Do
        If ft = "FILELIST" Then part = fn
                           Else part = fn || "." || ft
        End  /*  If  ..  Do  */
    If Translate(Strip(part),"_","/") = nick Then Do    /*   match!   */
        If rest = "" Then Return fn ft fm menu path
        If ft = "FILELIST" Then Do
            If  exist(fn 'FILELIST *') | ,
                exist(fn 'NAMES *') Then Return gresolve(fn rest)
                                        Else Return gresolve(menu rest)
                                        /* stay in same menu */
            End  /*  If  ..  Do  */
        emsg = "path" nick "is not a directory"
        Return fn ft fm menu path
        End  /*  If  ..  Do  */
    End  /*  Do  For  */

/*  At this point, the partial path (nick) has not resolved into a    *
 *  filename/filetype pair (from the FILELIST,  if any).   We return  *
 *  a value indicating that pipespec() should use only the path:      */

Return "? ? -" menu path



/* ------------------------------------------------------------ PIPESPEC
 * Return the correct pipeline specification for the file requested.
 * If,  for this file in this menu,  there is a "pipe" specification
 * in the NAMES file,  then simply return that string.
 */
PIPESPEC: Procedure Expose emsg client
Parse Upper Arg fn ft fm menu path

/*  Say " File:" fn ft fm  */
/*  Say " Menu:" menu  */
/*  Say " Path:" path  */
Parse Upper Var path nick "/" rest

If exist(menu "NAMES *") Then Do
    If fm = "-" Then Do
        'CALLPIPE CMS NAMEFIND :NICK' nick ':PIPE :FN :FT :AUTH' ,
                '(FILE' menu '| VAR PIPE | DROP' ,
                '| VAR OFN | DROP | VAR OFT | DROP | VAR AUTH'
        If rc = 32 Then
        'CALLPIPE CMS NAMEFIND :NICK *      :PIPE :FN :FT :AUTH' ,
                '(FILE' menu '| VAR PIPE | DROP' ,
                '| VAR OFN | DROP | VAR OFT | DROP | VAR AUTH'
        If rc ^= 0 Then Do; pipe = ""; auth = ""; End
        Else Do
            If fn = "?" Then fn = ofn
            If ft = "?" Then ft = oft
            End  /*  Else  ..  Do  */
        End  /*  If  ..  Do  */

    Else Do
        'CALLPIPE CMS NAMEFIND :FN' fn ':FT' ft ':PIPE :AUTH' ,
                '(FILE' menu '| VAR PIPE | DROP | VAR AUTH'
        If rc = 32 Then
        'CALLPIPE CMS NAMEFIND :FN  *   :FT' ft ':PIPE :AUTH' ,
                '(FILE' menu '| VAR PIPE | DROP | VAR AUTH'
        If rc = 32 Then
        'CALLPIPE CMS NAMEFIND :FN  *   :FT  *   :PIPE :AUTH' ,
                '(FILE' menu '| VAR PIPE | DROP | VAR AUTH'
        If rc ^= 0 Then Do; pipe = ""; auth = ""; End
        End  /*  Else  Do  */
    End  /*  If  ..  Do  */

Else Do; pipe = ""; auth = ""; End

If pipe = "" & ft = "FILELIST" & exist(fn "NAMES *") Then Do
    'CALLPIPE CMS NAMEFIND :NICK . :PIPE :AUTH (FILE' fn ,
            '| VAR PIPE | DROP | VAR AUTH'
    If rc ^= 0 Then Do; pipe = ""; auth = ""; End
    End  /*  If  ..  Do  */

If ^authorized(client,auth) Then pipe = "- access restricted."

If fn = "" Then fn = "?"
If ft = "" Then ft = "?"
If fm = "" Then fm = "?"
If pipe = "" Then pipe = "DISK" fn ft fm

Say "Pipe:" pipe
/* perform substitution */
/*  'CALLPIPE VAR PIPE | CHANGE /&0/' || path || '/ |',               *
 *                      'CHANGE /&1/' ||  fn  || '/ |',               *
 *                      'CHANGE /&2/' ||  ft  || '/ |',               *
 *                      'CHANGE /&3/' ||  fm  || '/ |',               *
 *                      'CHANGE /&4/' || menu || '/ | VAR PIPE'       */
Do While Index(rest,'/') > 0; Parse Var rest . '/' rest; End
p = Index(pipe,'&')
Do While p > 0 & Length(pipe) > p
    pk = Substr(pipe,p,2)
    Select  /*  pk  */
        When pk = "&0" Then
            pipe = Left(pipe,p-1) || path || Substr(pipe,p+2)
        When pk = "&1" Then
            pipe = Left(pipe,p-1) ||  fn  || Substr(pipe,p+2)
        When pk = "&2" Then
            pipe = Left(pipe,p-1) ||  ft  || Substr(pipe,p+2)
        When pk = "&3" Then
            pipe = Left(pipe,p-1) ||  fm  || Substr(pipe,p+2)
        When pk = "&4" Then
            pipe = Left(pipe,p-1) || menu || Substr(pipe,p+2)
        Otherwise Say "Token" pk "not replaced"
        End  /*  Select  pk  */
    p = Index(pipe,'&',p+1)
    End  /*  Do  While  */

If pipe = "" Then Do
    If Length(ft) > 4 & Left(ft,4) = "HELP" Then
        pipe = 'COMMAND HELP' Substr(ft,5) fn '(NOSCREEN'
    Else pipe =  "DISK" fn ft fm
    End  /*  If  ..  Do  */

Return pipe



/* --------------------------------------------------------------- EXIST
 * This function returns TRUE if the file specified exists.
 */
EXIST:    Procedure
Parse Arg fn ft fm .
Say "EXIST: looking for" fn ft fm
'CALLPIPE COMMAND STATE' fn ft fm '| HOLE'
Return (rc = 0)



/* ---------------------------------------------------------- AUTHORIZED
 * Verify that the client is allowed access, based on IP addr or name.
 * (we originally used an internal routine called IPA2C(),  but Arty's
 * RXSOCKET provides an Inet_addr() function,  which is now preferred)
 */
AUTHORIZED: Procedure
Parse Upper Arg Client,AuthList
If AuthList = "" Then Return 1
ClientAddr = Inet_addr(Client)

rc = Socket('GetHostByAddr', ClientAddr, 'ClientName')
If rc = "-1" Then ClientName = Client
Say "Client looks like" ClientName Inet_ntoa(ClientAddr)
Upper ClientName

allow = 1
Do While AuthList ^= ""

    Parse Var AuthList Auth AuthList
    If Auth = "DENY" Then Do
        allow = 0; Parse Var AuthList Auth AuthList
        End  /*  If  ..  Do  */
    If Auth = "ALLOW" Then Do
        allow = 1; Parse Var AuthList Auth AuthList
        End  /*  If  ..  Do  */
    Say "Auth" Auth
    If Auth = '*' Then Return allow

    Parse Var Auth Auth "," AuthMask "," .
    AuthAddr = Inet_addr(Auth)
    If AuthAddr = "-1" Then Do              /* process as a name spec */
        If Left(Auth,1) = '.' Then Do
            TruncName = Right(ClientName,Length(Auth))
            Say "Truncated ClientName" TruncName
            If TruncName = Auth Then Return allow
            End  /*  If  ..  Do  */
        Else If ClientName = Auth Then Return allow
        End  /*  If  ..  Do  */

    Else Do                                 /* must be a numeric spec */
        AuthMask = Inet_addr(AuthMask)
        If AuthMask = "-1" Then Do
            AuthMask = bitand(AuthAddr,'C0000000'x)
            Select  /*  AuthMask  */
                When AuthMask = '00000000'x Then AuthMask = 'FF000000'x
                When AuthMask = '80000000'x Then AuthMask = 'FFFF0000'x
                When AuthMask = 'C0000000'x Then AuthMask = 'FFFFFF00'x
                End  /*  Select  AuthMask  */
            End  /*  If  ..  Do  */
        Say "ADDR" c2x(AuthAddr) "MASK" c2x(AuthMask)
        AuthAddr = bitxor(AuthAddr,ClientAddr)
        AuthAddr = bitand(AuthAddr,AuthMask)
        If AuthAddr = '00000000'x Then Return allow
        End  /*  Else  Do  */

    End  /*  Do  While  */

Return 0

