/*
 *        Name: GOPHERDM REXX
 *              CMS Gopher Server menu builder pipeline stage
 *      Author: Rick Troth, Rice Univ, I/S VM Systems Support
 *        Date: 1992-Apr-21
 */

Parse Arg localhost localport menu gpath
gpath = Strip(gpath)
If gpath = "/" Then gpath = ""

'READTO LINE'
Do While rc = 0
    key = Left(line,1)
    Select  /*  key  */
        When key = "*" Then nop
        When key = " " Then Call FILELIST line
        Otherwise           Call PREFAB line
        End  /*  Select  key  */
    'READTO LINE'
    End  /*  Do  For  */

Return



/* ------------------------------------------------------------ FILELIST
 * We're reading (apparently) a plain FILELIST file.   (good)
 */
FILELIST:

Parse Arg fn ft fm rpath '05'x name
If ft = "" Then Return
If fm = "" Then fm = "*"
rpath = Strip(rpath)
If rpath = "" Then Select /* ft */
    When ft = "FILELIST" Then rpath = locase(fn)
    Otherwise                 rpath = locase(fn||"."||ft)
    End  /*  Select  ft  */
name = Strip(name)
If name = "" Then name = rpath          /* default name is rel path */
rpath = Translate(rpath,"_","/")        /* no slashes in path part */
host = localhost
port = localport
type = gtype(ft)
path = type || gpath || "/" || rpath

/* read Gopher link information if this is a GOPHER file */
If ft = "GOPHER" Then Call GLINK fn ft fm

/* read overrides from the NAMES file, if any */
Call GOVER fn ft, menu

'OUTPUT' type || name || '05'x || ,
      /* type */ path || '05'x || ,
                 host || '05'x || port

Return



/* -------------------------------------------------------------- PREFAB
 * Preceeding stage has supplied some of the parameters.   (thank you)
 */
PREFAB:

Parse Arg uname '05'x upath '05'x uhost '05'x uport '05'x urest
Parse Var upath 1 utype 2 upath
upath = utype || gpath || "/" || upath
If uhost = "" Then uhost = localhost
If uport = "" Then uport = localport
'OUTPUT' uname || '05'x || ,
         upath || '05'x || ,
         uhost || '05'x || ,
         uport || '05'x urest
Return



/* --------------------------------------------------------------- GLINK
 * Read the specified Gopher link definition file.
 */
GLINK:    Procedure Expose type name path host port gpath rpath
Parse Arg fn ft fm .
'CALLPIPE DISK' fn ft fm '| STEM STEM.'

path = ""; port = ""

Do i = 1 To stem.0
    Parse Var stem.i var '=' val
    Upper var; var = Strip(var)
    Select  /*  var  */
        When var = "TYPE" Then type = Strip(val)
        /*                     path = type || gpath || "/" || rpath   */
        When var = "NAME" Then name = Strip(val)
        When var = "PATH" Then path = Strip(val)
        When var = "HOST" Then host = Strip(val)
        When var = "PORT" Then port = Strip(val)
        Otherwise nop
        End  /*  Select  var  */
    End  /*  Do  For  */

Return



/* --------------------------------------------------------------- GOVER
 * Read the specified NAMES file for menu data overrides, if any.
 */
GOVER:    Procedure Expose type name path host port gpath rpath ,
                           localhost
Parse Upper Arg file,menu
If menu = "" Then Return
Parse Var file fn ft fm .
If ft = "" Then Return

'CALLPIPE CMS NAMEFIND :FN' fn ':FT' ft ,
        ':TYPE :NAME :PATH :HOST :PORT (FILE' menu ,
        '| VAR OTYPE | DROP' ,
        '| VAR ONAME | DROP' ,
        '| VAR OPATH | DROP' ,
        '| VAR OHOST | DROP' ,
        '| VAR OPORT'

If rc ^= 0 Then Return

/*      defaults for  type, name, path, host, port    *
 *      are supplied by the calling routine           */

If oport ^= "" Then port = oport
If ohost ^= "" Then host = ohost
If opath ^= "" Then path = opath
If oname ^= "" Then name = oname
If otype ^= "" Then type = otype

Select
    When ohost ^= "" & oport = "" Then port = 70
    When ohost = "" & oport ^= "" Then host = localhost
    Otherwise nop
    End  /*  When  Select  */

If ohost ^= "" | oport ^= "" Then Do
    path = opath
    type = otype
    If type = "" Then type = "1"
    End  /*  If  ..  Do  */
Else If otype ^= "" & opath = "" Then
    path = otype || gpath || "/" || rpath

Return



/* --------------------------------------------------------------- GTYPE
 * Returns the default Gopher type character for CMS filetype specified.
 */
GTYPE:    Procedure
Parse Arg ft,.
Select  /*  ft  */
    When ft = "FILELIST" Then Return "1"
    When ft = "GOPHER"   Then Return "1"
/*  When ft = "CSO"      Then Return "2"  *
 *  When ft = ?          Then Return "3"  *
 *  When ft = "MAC"      Then Return "4"  *
 *  When ft = "DOS"      Then Return "5"  *
 *  When ft = "UUE"      Then Return "6"  *
 *  When ft = "SEARCH"   Then Return "7"  *
 *  When ft = "TELNET"   Then Return "8"  */
    When ft = "TAR"      Then Return "9"
    When ft = "SND"      Then Return "s"
    When ft = "EPS"      Then Return "p"
    When ft = "PS"       Then Return "p"
    When ft = "MODULE"   Then Return "r"
    When ft = "CARDDUMP" Then Return "r"
    Otherwise            Return "0"
    End  /*  Select  ft  */



/* -------------------------------------------------------------- LOCASE
 */
LOCASE:   Procedure
Parse Arg s
lc = "abcdefghijklmnopqrstuvwxyz"
uc = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
Return Translate(s,lc,uc)

